<?php

declare(strict_types=1);

namespace Drupal\social_auth_keycloak\Plugin\Network;

use Drupal\Core\Http\ClientFactory;
use Drupal\Core\Url;
use Drupal\social_api\SocialApiException;
use Drupal\social_auth\Plugin\Network\NetworkBase;
use Drupal\social_auth\Settings\SettingsInterface;
use Drupal\social_auth_keycloak\Settings\KeycloakAuthSettingsInterface;
use Stevenmaguire\OAuth2\Client\Provider\Keycloak;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines Social Auth Keycloak Network Plugin.
 *
 * @Network(
 *   id = "social_auth_keycloak",
 *   short_name = "keycloak",
 *   social_network = "Keycloak",
 *   img_path = "img/keycloak_icon_256px.svg",
 *   type = "social_auth",
 *   class_name = "\Stevenmaguire\OAuth2\Client\Provider\KeycloakResourceOwner",
 *   auth_manager = "\Drupal\social_auth_keycloak\KeycloakAuthManager",
 *   routes = {
 *     "redirect": "social_auth.network.redirect",
 *     "callback": "social_auth.network.callback",
 *     "settings_form": "social_auth.network.settings_form",
 *   },
 *   handlers = {
 *     "settings": {
 *       "class": "Drupal\social_auth_keycloak\Settings\KeycloakAuthSettings",
 *       "config_id": "social_auth_keycloak.settings"
 *     },
 *   },
 * )
 */
final class KeycloakAuth extends NetworkBase {

  private ClientFactory $httpClientFactory;

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->httpClientFactory = $container->get('http_client_factory');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function initSdk(): ?Keycloak {
    $class_name = Keycloak::class;
    if (!\class_exists($class_name)) {
      throw new SocialApiException(\sprintf('The Keycloak library for PHP League OAuth2 not found. Class: %s.', $class_name));
    }

    if (!$this->validateConfig($this->settings)) {
      return NULL;
    }

    $settings = $this->settings;
    \assert($settings instanceof KeycloakAuthSettingsInterface);

    // All these settings are mandatory.
    $league_settings = [
      'authServerUrl' => $settings->getServerUrl(),
      'realm' => $settings->getRealm(),
      'clientId' => $settings->getClientId(),
      'clientSecret' => $settings->getClientSecret(),
      'redirectUri' => $this->getCallbackUrl()->setAbsolute()->toString(),
    ];

    if ($settings->hasEncryption()) {
      $league_settings['encryptionAlgorithm'] = $settings->getEncryptionAlgorithm();
      $league_settings['encryptionKey'] = $settings->getEncryptionKey();
    }

    // Proxy configuration data for outward proxy.
    $proxy_config = $this->httpClientFactory->fromOptions()->getConfig('proxy');
    if (\array_key_exists('https', $proxy_config)) {
      $league_settings['proxy'] = $proxy_config['https'];
    }
    elseif (\array_key_exists('proxy', $league_settings) && \array_key_exists('http', $proxy_config)) {
      $league_settings['proxy'] = $proxy_config['http'];
    }

    return new Keycloak($league_settings);
  }

  /**
   * Checks that module is configured.
   *
   * @return bool
   *   True if module is configured, false otherwise.
   */
  protected function validateConfig(SettingsInterface $settings): bool {
    \assert($this->settings instanceof KeycloakAuthSettingsInterface);
    $callables = [
      [$settings, 'getServerUrl'],
      [$settings, 'getRealm'],
      [$settings, 'getClientId'],
      [$settings, 'getClientSecret'],
    ];
    foreach ($callables as $callable) {
      if ($callable() === NULL) {
        $this->loggerFactory
          ->get('social_auth_keycloak')
          ->error('Define %key in module settings.', ['%key' => substr(end($callable), 3)]);

        return FALSE;
      }
    }
    if ($settings->hasEncryption() && $this->settings->getEncryptionKey() === NULL) {
      $this->loggerFactory
        ->get('social_auth_keycloak')
        ->error('Define an encryption key in module settings.');

      return FALSE;
    }

    return TRUE;
  }

}
